import time
import wifi
import socketpool
import ssl
import adafruit_minimqtt.adafruit_minimqtt as MQTT
import json


# Define callback methods which are called when events occur
def connect(client, userdata, flags, rc):
    # This function will be called when the client is connected
    # successfully to the broker.
    print("Connected to MQTT Broker!")
    print("Flags: {0}\n RC: {1}".format(flags, rc))


def disconnect(client, userdata, rc):
    # This method is called when the client disconnects
    # from the broker.
    print("Disconnected from MQTT Broker!")


def subscribe(client, userdata, topic, granted_qos):
    # This method is called when the client subscribes to a new feed.
    print("Subscribed to {0} with QOS level {1}".format(topic, granted_qos))


def unsubscribe(client, userdata, topic, pid):
    # This method is called when the client unsubscribes from a feed.
    print("Unsubscribed from {0} with PID {1}".format(topic, pid))


def publish(client, userdata, topic, pid):
    # This method is called when the client publishes data to a feed.
    print("Published to {0} with PID {1}".format(topic, pid))


def message(client, topic, message):
    # This method is called when a topic the client is subscribed to
    # has a new message.
    print(f"New message on topic {topic}: {message}")


pool = socketpool.SocketPool(wifi.radio)
ssl_context = ssl.create_default_context()

# Set up a MiniMQTT Client
# NOTE: We'll need to connect insecurely for ethernet configurations.
mqtt_client = MQTT.MQTT(
    broker="192.168.x.x",
    port=1883,
    username="",
    password="",
    is_ssl=False,
    socket_pool=pool,
    ssl_context=ssl_context,
)

# Connect callback handlers to client
mqtt_client.on_connect = connect
mqtt_client.on_disconnect = disconnect
mqtt_client.on_subscribe = subscribe
mqtt_client.on_unsubscribe = unsubscribe
mqtt_client.on_publish = publish
mqtt_client.on_message = message

# MQTT Topic
# Use this topic if you'd like to connect to a standard MQTT broker
mqtt_topic = "test/topic"

print("Attempting to connect to %s" % mqtt_client.broker)
mqtt_client.connect()

# print("Subscribing to %s" % mqtt_topic)
# mqtt_client.subscribe(mqtt_topic)

# print("Publishing to %s" % mqtt_topic)d
# mqtt_client.publish(mqtt_topic, "Hello Broker!")

# print("Unsubscribing from %s" % mqtt_topic)
# mqtt_client.unsubscribe(mqtt_topic)

# print("Disconnecting from %s" % mqtt_client.broker)
# mqtt_client.disconnect()

import board
import busio

i2c = busio.I2C(scl=board.GPIO7, sda=board.GPIO6)
assert i2c.try_lock()
print(i2c.scan())
i2c.unlock()


# 测温测试
if 0:
    import adafruit_bmp280

    bmp280 = adafruit_bmp280.Adafruit_BMP280_I2C(i2c, address=0x76)
    bmp280.sea_level_pressure = 1013.25

    import digitalio

    led = digitalio.DigitalInOut(board.GPIO15)
    led.direction = digitalio.Direction.OUTPUT
    led.value = True

    mqtt_client.subscribe("test/cmd")

    def func(client, topic, message):
        led.value = int(message)
        print(f"New message on topic {topic}: {message}")

    mqtt_client.on_message = func

    while True:
        # Poll the message queue
        mqtt_client.loop(timeout=1)

        msg = {"Temperature": bmp280.temperature}
        # Send a new message
        mqtt_client.publish(mqtt_topic, json.dumps(msg))
        print(msg)
        time.sleep(1)


# 测距测试
if 0:
    import adafruit_vl53l0x

    vl53 = adafruit_vl53l0x.VL53L0X(i2c)
    import adafruit_us100

    # Connect TX to TX, RX to RX
    uart = busio.UART(board.GPIO4, board.GPIO5, baudrate=9600)
    sonar = adafruit_us100.US100(uart)

    while True:
        # Poll the message queue
        mqtt_client.loop(timeout=1)

        msg = {"vl53l0x": vl53.range, "sonar": int(sonar.distance * 10)}
        # Send a new message
        mqtt_client.publish(mqtt_topic, json.dumps(msg))
        print(msg)
        time.sleep(1)


# 舵机测试
if 1:
    import pwmio
    from adafruit_motor import servo

    pwm = pwmio.PWMOut(board.GPIO14, frequency=50)
    s1 = servo.Servo(pwm, min_pulse=500, max_pulse=2500)

    mqtt_client.subscribe("test/cmd")

    def func(client, topic, message):
        print(f"New message on topic {topic}: {message}")
        s1.angle = int(message)
        msg = {"angle": int(message)}
        mqtt_client.publish(mqtt_topic, json.dumps(msg))

    mqtt_client.on_message = func

    while True:
        # Poll the message queue
        mqtt_client.loop(timeout=1)
